<?php

namespace Drupal\entity_usage_explorer;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Twig\Extension\AbstractExtension;
use Twig\TwigFunction;

/**
 * Provides custom Twig functions for the Entity Usage Explorer module.
 */
class UsageHelper extends AbstractExtension {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The entity usage service.
   *
   * @var \Drupal\entity_usage_explorer\UsageService
   */
  protected UsageService $entityUsageService;

  /**
   * Constructs a UsageHelper object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\entity_usage_explorer\UsageService $entity_usage_service
   *   The entity usage service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, UsageService $entity_usage_service) {
    $this->entityTypeManager = $entity_type_manager;
    $this->entityUsageService = $entity_usage_service;
  }

  /**
   * {@inheritdoc}
   */
  public function getFunctions(): array {
    return [
      new TwigFunction('loadParagraphParent', [$this, 'loadParagraphParent']),
      new TwigFunction('getEntityLabel', [$this, 'getEntityLabel']),
      new TwigFunction('getCanonicalPath', [$this, 'getCanonicalPath']),
    ];
  }

  /**
   * Loads the parent entity of a paragraph.
   *
   * @param int $pid
   *   The paragraph ID.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   The parent entity if found, NULL otherwise.
   */
  public function loadParagraphParent(int $pid): ?EntityInterface {
    $paragraph = $this->entityTypeManager->getStorage('paragraph')->load($pid);

    if ($paragraph instanceof EntityInterface && method_exists($paragraph, 'getParentEntity')) {
      return $paragraph?->getParentEntity();
    }

    return NULL;
  }

  /**
   * Returns the value of 'label' entity key of entity type.
   *
   * @param string $entity_type
   *   The entity type name.
   * @param int $entity_id
   *   The entity id.
   *
   * @return string|null
   *   'label' entity key value if exists, NULL otherwise.
   */
  public function getEntityLabel(string $entity_type, int $entity_id): ?string {
    $content_entity_definitions = $this->entityUsageService->loadContentEntityTypes(TRUE);
    $label_key = $content_entity_definitions[$entity_type]['label_key'];
    $entity = $this->entityTypeManager->getStorage($entity_type)->load($entity_id);
    return $label_key ? $entity->{$label_key}->value : NULL;
  }

  /**
   * Returns the canonical path of entity.
   *
   * @param string $entity_type
   *   The entity type name.
   * @param int $entity_id
   *   The entity id.
   *
   * @return string|null
   *   Canonical path if exists, NULL otherwise.
   */
  public function getCanonicalPath(string $entity_type, int $entity_id): ?string {
    $content_entity_definitions = $this->entityUsageService->loadContentEntityTypes(TRUE);
    $canonical_url = $content_entity_definitions[$entity_type]['canonical_url'];
    return $canonical_url ? str_replace("{{$entity_type}}", $entity_id, $canonical_url) : NULL;
  }

}
